package de.ugoe.cs.rwm.mocci;

import java.nio.file.Path;

import org.eclipse.cmf.occi.core.Configuration;
import org.eclipse.cmf.occi.core.Link;
import org.eclipse.cmf.occi.infrastructure.Compute;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.modmacao.occi.platform.Component;

import de.ugoe.cs.rwm.docci.ModelUtility;
import de.ugoe.cs.rwm.docci.connector.Connector;
import monitoring.Monitorableproperty;

public class DownScaler extends AbsScaler {

	public DownScaler(Connector conn, Path runtimePath) {
		this.conn = conn; 
		this.runtimePath = runtimePath;
	}
	
	public Resource downScaleNodes() {
		EList<EObject> toDelete = new BasicEList<EObject>();
		runtimeModel = ModelUtility.loadOCCIintoEMFResource(conn.loadRuntimeModel(runtimePath));
		Configuration config = ((Configuration) runtimeModel.getContents().get(0));
		boolean downScale = false;
		
		for(org.eclipse.cmf.occi.core.Resource res : config.getResources()) {
			if(res instanceof Compute) {
				Compute comp = (Compute) res;
				if(comp.getTitle().contains("hadoop-worker-additional")) {
					Monitorableproperty monProp = getAttachedCPUMonProp(comp);
					if(monProp != null && monProp.getMonitoringResult() != null) {
						if(monProp.getMonitoringResult().equals("None")) {
							System.out.println("      VM with None CPU utilization found: " + comp.getId());
							toDelete.add(comp);
							toDelete.addAll(linksAndComponents(comp));
							System.out.println("      Deleting Entities Around: " + comp.getTitle() +" ("+comp.getId()+")");
							downScale = true;
							break;
						}
					}
				}			
			}
		}
		
		if(downScale == false) {
			System.out.println("      Every Compute busy/Only one worker! Skipping downScale!");
		}
		
		runtimeModel.getContents().removeAll(toDelete);
		return runtimeModel;
		
	}

	private Monitorableproperty getAttachedCPUMonProp(Compute comp) {
		for(Link link: comp.getRlinks()) {
			//System.out.println("LINK: " + link);
			if(link instanceof  Monitorableproperty) {
				Monitorableproperty monProp = (Monitorableproperty) link;
				//System.out.println("MonPROP: " + monProp);
				//System.out.println("Prop: " + monProp.getMonitoringProperty());
				if(monProp.getMonitoringProperty().equals("CPU")) {
					return monProp;
				}
			}
		}
		return null;
	}

	private EList<EObject> linksAndComponents(Compute comp) {
		EList<EObject> toDelete = new BasicEList<EObject>();
		toDelete.addAll(comp.getLinks());
		toDelete.addAll(comp.getRlinks());
		
		for(Link link: comp.getRlinks()) {
			if(link.getSource() instanceof Component) {
				toDelete.add(link.getSource());
			}
			if(link instanceof Monitorableproperty) {
				toDelete.add(link.getSource());
			}
		}

		return toDelete;
	}

}
